local M=class(function(self)
  self._children={}
end)

local function callchildren(self, fname, ...)
  for k,v in ipairs(self._children) do
    if v[fname] then v[fname](v, ...) end
  end
end

local function callfunction(obj, fname, ...)
  if obj[fname] then
    obj[fname](...)
  end
end

function M:update(dt)
  callchildren(self, 'update', dt)
end

function M:draw()
  callchildren(self, 'draw')
end

function M:keypressed(a, b)
  callchildren(self, 'keypressed', a, b)
end

function M:keyreleased(a, b)
  callchildren(self, 'keyreleased', a, b)
end

function M:mousepressed(x, y, b)
  callchildren(self, 'mousepressed', x, y, b)
end

function M:mousereleased(x, y, b)
  callchildren(self, 'mousereleased', x, y, b)
end

function M:load()
  callchildren(self, 'load')
end

function M:add(obj)
  table.insert(self._children, obj)
end

function M:remove(obj)
  if type(obj)=='number' then
    return table.remove(self._children, obj)
  else
    for k,v in ipairs(self._children) do
      if v==obj then
        table.remove(self._children, k)
        return v
      end
    end
  end
end

function M:getChildren()
  return self._children
end

return M
